<?php
App::uses('EvBasketAppModel', 'EvBasket.Model');
/**
 * BasketItem Model
 *
 * @property Basket $Basket
 * @property Model $Model
 */
class BasketItem extends EvBasketAppModel {

	/**
	 * Validation rules
	 *
	 * @var array
	 */
	public $validate = array(
		'basket_id' => array(
			'numeric' => array(
				'rule' => array('numeric'),
				'on' => 'create'
			),
		),
		'model' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				'on' => 'create'
			),
		),
		'model_id' => array(
			'numeric' => array(
				'rule' => array('numeric'),
				'on' => 'create'
			),
		),
		'unit_price' => array(
			'numeric' => array(
				'rule' => array('numeric')
			),
		),
		'quantity' => array(
			'numeric' => array(
				'rule' => array('numeric')
			),
		)
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

	/**
	 * belongsTo associations
	 *
	 * @var array
	 */
	public $belongsTo = array(
		'Basket' => array(
			'className' => 'EvBasket.Basket'
		)
	);

	/**
	 * constructor - setup the dynamic relationships
	 *
	 */
	public function __construct($id = false, $table = null, $ds = null) {
		parent::__construct($id, $table, $ds);

		$relationships = Configure::read('EvBasket.BasketItemBelongsTo');
		if (! empty($relationships)) {
			$belongsTo = array();
			foreach ($relationships as $key => $value) {
				$belongsTo[$key] = $value;
			}

			if (! empty($belongsTo)) {
				$this->bindModel(
					array(
						'belongsTo' => $belongsTo
					),
					false
				);
			}
		}
	}

	public function afterFind($results, $primary = false) {
		$results = parent::afterFind($results, $primary);

		if ($primary === true) {
			$results = Hash::combine(
				$results,
				array(
					'%s.%s',
					'{n}.BasketItem.model',
					'{n}.BasketItem.model_id'
				),
				'{n}'
			);
		}

		return $results;
	}

	/**
	 * add an item to the basket
	 *
	 * @param 	int 		$basketId 		ID of the basket row
	 * @param 	array 		$item 			Array with two elements of model and model_id
	 * @param 	decimal 	$unitPrice 		Unit Price of the item
	 * @param 	int 		$quantity		Number of items to add
	 * @return 	bool
	 */
	public function addItem($basketId, $item, $unitPrice, $quantity) {
		$this->clear();

		$this->set(
			array(
				'basket_id' => $basketId,
				'model' => $item['model'],
				'model_id' => $item['model_id'],
				'unit_price' => $unitPrice,
				'quantity' => $quantity,
				'row_total' => ($unitPrice * $quantity)
			)
		);

		return (bool)$this->save();
	}

	/**
	 * update an item in the basket
	 *
	 * @param 	array 		$basketItem 	The existing BasketItem Row
	 * @param 	int 		$quantity		Number of items to add
	 * @param 	decimal 	$unitPrice 		Unit Price of the item
	 * @return 	bool
	 */
	public function updateItem($basketItem, $newQuantity, $unitPrice = null) {
		$this->clear();

		// if we're not updating the unit price use old one
		if (empty($unitPrice)) {
			$unitPrice = $basketItem['unit_price'];
		}

		$this->set(
			array(
				'id' => $basketItem['id'],
				'unit_price' => $unitPrice,
				'quantity' => $newQuantity,
				'row_total' => ($unitPrice * $newQuantity)
			)
		);

		return (bool)$this->save();
	}

	/**
	 * delete an item from the basket
	 *
	 * @param 	array 		$basketItem 	The existing BasketItem Row
	 * @return 	bool
	 */
	public function deleteItem($basketItem) {
		$this->clear();

		return $this->delete($basketItem['id']);
	}
}
